def main():
    import argparse
    import os
    import datetime
    import frontmatter
    import tomli
    import shutil
    from .generator import generator
    from .httpserver import bestatic_serv
    from .quickstart import quickstart
    from .newcontent import newpost, newpage
    import bestatic

    parser = argparse.ArgumentParser(description="Program to accept command-line arguments in Bestatic")
    parser.add_argument("action", nargs="?", choices=["generator", "quickstart", "version", "newpage", "newpost"], default="generator",
                        help="Specify whether you are checking a version (argument: version),"
                             " initializing the site (argument: quickstart), or doing a build (argument: generator)."
                             " If only package name is entered, by default, we will assume that you are just trying to"
                             " build your website into '_output' directory")

    parser.add_argument("filename", nargs="?", help="For newpost or newpage argument, specify the filename(.md will "
                                                    "be added automatically)")
    parser.add_argument("--directory", "-d", help="Specify a custom directory. If not specified,"
                                                  " it will generate the website in '_output' directory by default")
    parser.add_argument("--theme", "-t", help="Specify the theme. If not specified, it will use the 'Amazing' theme.")
    parser.add_argument("--serve", "-s", action="store_true", help="Start server after the build...")

    args = parser.parse_args()

    if args.action == "quickstart":
        if args.theme:
            config = quickstart(args.theme)
        else:
            config = quickstart()

        generator(**config)

        config = quickstart()
        generator(**config)
    elif args.action == "version":
        print(f'Bestatic version: {bestatic.__version__}')
    elif args.action == "newpost":
        if not args.filename:
            raise ValueError("Please specify filename to use 'newpost' function.")
        else:
            os.chdir(os.getcwd())
            newpost(args.filename)
    elif args.action == "newpage":
        if not args.filename:
            raise ValueError("Please specify filename to use 'newpage' function.")
        else:
            os.chdir(os.getcwd())
            newpage(args.filename)
    else:
        with open("config.toml", mode="rb") as ft:
            config = tomli.load(ft)
        if args.theme:
            config["theme"] = args.theme

        theme = config["theme"]
        current_directory = os.getcwd()
        path = os.path.join(current_directory, "themes", theme)
        is_exist = os.path.exists(path)
        if not is_exist:
            raise FileNotFoundError(f"Theme directory does not exist! Please make sure a proper theme is present in themes directory")
        generator(**config)

    if args.directory:
        shutil.rmtree(args.directory) if os.path.exists(args.directory) else None
        os.rename("_output", args.directory)

    if args.serve:
        bestatic_serv(args.directory) if args.directory else bestatic_serv()


if __name__ == '__main__':
    main()



def quickstart(*theme):
    import os
    import tomli
    import tomli_w
    import frontmatter
    import datetime

    theme = "Amazing" if not theme else theme

    current_directory = os.getcwd()
    path = os.path.join(current_directory, "themes", theme)
    is_exist = os.path.exists(path)

    if not is_exist:
        raise FileNotFoundError(f"Theme directory does not exist! Please make sure a proper theme is present in themes directory")

    config_dict = {
        "siteURL": "http://example.org",
        "title": input("Enter the title of the website: "),
        "description": input("Enter the description of the website: "),
        "theme": theme,
        "number_of_pages": 1,
        "ugly_url": False
    }

    while not config_dict["title"]:
        print("Title of the website cannot be empty! Please try again. ")
        config_dict["title"] = input("Enter the title of the website: ")

    while not config_dict["description"]:
        print("Description of the website cannot be empty! Please try again. ")
        config_dict["description"] = input("Enter the description of the website: ")

    output_file = "config.toml"

    # Write the dictionary to the TOML file
    with open(output_file, "wb") as f:
        tomli_w.dump(config_dict, f)

    with open("config.toml", mode="rb") as ft:
        config = tomli.load(ft)

    post_dict = {
        "post1": {'title': 'My First Post',
                  'date': datetime.datetime.now().strftime("%B %d, %Y"),
                  'tags': "first, dolor-ipsum, sit, amet",
                  'description': "The first post of the Blog",
                  'slug': 'the-first-article'},
        "post2": {'title': 'My Second Post',
                  'date': datetime.datetime.now().strftime("%B %d, %Y"),
                  'tags': "good, dolor-ipsum, sit, great",
                  'description': "The second post of the Blog",
                  'slug': 'the-second-article'},
    }

    page_dict = {
        "page1": {
            'title': 'About',
            'description': "About the writer of this website",
            'slug': 'about'
        },
        "page2": {
            'title': 'Contact',
            'description': "About the contact",
            'slug': 'contact'
        }
    }

    # Create new markdown files
    markdown_content_post = "Well, hello there, world. This is an example post"
    markdown_content_page = "This is an example page. Thanks for visiting this website!"

    post = {}
    page = {}
    # Add the frontmatter to the markdown content
    for item in post_dict:
        post[item] = frontmatter.Post(markdown_content_post, **post_dict[item])

    for item in page_dict:
        page[item] = frontmatter.Post(markdown_content_page, **page_dict[item])
    # page = frontmatter.Post(markdown_content_page, **page_dict)

    os.makedirs("posts", exist_ok=True)
    os.makedirs("pages", exist_ok=True)

    # Save the updated contents to files
    for item in post:
        with open(f'posts/{post[item].metadata["slug"]}.md', 'w', encoding='utf-8') as f:
            f.write(frontmatter.dumps(post[item]))

    for item in page:
        with open(f'pages/{page[item].metadata["slug"]}.md', 'w', encoding='utf-8') as f:
            f.write(frontmatter.dumps(page[item]))

    return config


if __name__ == '__main__':
    quickstart()



import os
import datetime
import xml.etree.ElementTree as ET


def get_last_modified_time(file_path):
    return datetime.datetime.fromtimestamp(os.path.getmtime(file_path)).isoformat()


def generate_sitemap(base_url, folder_path):
    root = ET.Element("urlset")
    root.set("xmlns", "http://www.sitemaps.org/schemas/sitemap/0.9")

    for currpath, dirpath, filenames in os.walk(folder_path):
        for filename in filenames:
            file_path = os.path.join(currpath, filename)
            if currpath == folder_path:
                file_name = filename if filename != "index.html" else ""
            elif currpath == os.path.join(folder_path, "post"):
                file_name = "post" + "/" + filename
            elif currpath == os.path.join(folder_path, "post", "tags"):
                file_name = "post" + "/" + "tags" + "/" + filename
            else:
                continue

            if "index.json" in file_name or "404" in file_name or ".xml" in file_name:
                continue
            else:
                url = ET.SubElement(root, "url")
                loc = ET.SubElement(url, "loc")
                loc.text = f"{base_url}/{file_name}"
                lastmod = ET.SubElement(url, "lastmod")
                lastmod.text = get_last_modified_time(file_path)

            # ET.SubElement(url, "newline")

    tree = ET.ElementTree(root)
    tree.write("_output/sitemap.xml", encoding="utf-8", xml_declaration=True)


if __name__ == "__main__":
    base_url = "https://example.org"
    folder_path = "_output"
    generate_sitemap(base_url, folder_path)




import os
import datetime
import frontmatter


def newpost(filename):
    post_dict = {'title': "This is a sample post",
                 'date': datetime.datetime.now().strftime("%B %d, %Y"),
                 'tags': "tag1, tag2",
                 'description': "This is sample description or summary of a post",
                 'slug': "enter something here to generate URL or delete this 'slug' field to autogenerate"
                 }

    markdown_content_post = ("This is an example post content in simple markdown. Please replace this with content of "
                             "your choice. ")

    post = frontmatter.Post(markdown_content_post, **post_dict)
    os.makedirs("posts", exist_ok=True)

    with open(f'posts/{filename}.md', 'w', encoding='utf-8') as f:
        f.write(frontmatter.dumps(post))


def newpage(filename):
    page_dict = {'title': "This is a sample page",
                 'description': "This is sample description or summary of a page",
                 'slug': "enter something here to generate URL or delete this 'slug' field to autogenerate"
                 }

    markdown_content_post = ("This is an example page content in simple markdown. Please replace this with content of "
                             "your choice. ")

    page = frontmatter.Post(markdown_content_post, **page_dict)
    os.makedirs("pages", exist_ok=True)

    with open(f'pages/{filename}.md', 'w', encoding='utf-8') as f:
        f.write(frontmatter.dumps(page))



from setuptools import setup, find_packages
from bestatic import __version__

with open("README.md", "r") as f:
    description = f.read()

setup(
    name='Bestatic',
    version=__version__,
    author='Tatsat Banerjee',
    packages=find_packages(),
    install_requires=[
        'beautifulsoup4 == 4.12.3',
        'Jinja2 == 3.1.3',
        'Markdown == 3.5.2',
        'MarkupSafe == 2.1.5',
        'Pygments == 2.17.2',
        'python-frontmatter == 1.1.0',
        'PyYAML == 6.0.1',
        'soupsieve == 2.5',
        'text-unidecode == 1.3',
        'tomli == 2.0.1',
        'tomli_w == 1.0.0',
        'pytz == 2024.1',
        'feedgen == 1.0.0'
    ],
    python_requires=">=3.8",
    entry_points={
        'console_scripts': [
            'bestatic = bestatic.bestatic:main'
        ]
    },
    long_description=description,
    long_description_content_type="text/markdown",
)



def bestatic_serv(*directory):
    import http.server
    import socketserver

    PORT = 8080
    DIRECTORY = directory[0] if directory else "_output"

    class Handler(http.server.SimpleHTTPRequestHandler):
        extensions_map = {
            '': 'text/html',
            '.manifest': 'text/cache-manifest',
            '.html': 'text/html',
            '.png': 'image/png',
            '.jpg': 'image/jpg',
            '.svg': 'image/svg+xml',
            '.css': 'text/css',
            '.js': 'application/x-javascript',
            '.wasm': 'application/wasm',
            '.json': 'application/json',
            '.xml': 'application/xml',
        }

        def __init__(self, *args, **kwargs):
            super().__init__(*args, directory=DIRECTORY, **kwargs)

        def end_headers(self):
            self.send_my_headers()
            http.server.SimpleHTTPRequestHandler.end_headers(self)

        def send_my_headers(self):
            self.send_header("Cache-Control", "no-cache, no-store, must-revalidate")
            self.send_header("Pragma", "no-cache")
            self.send_header("Expires", "0")

        # def do_GET(self):
        #     if self.path == '/':
        #         self.path = '/index.html'
        #     return Handler.do_GET(self)

    with socketserver.TCPServer(("", PORT), Handler) as httpd:
        print("Serving at port", PORT)
        print("Click http://localhost:8080 to visit the live website")
        print("Click Ctrl+C to shut down the server")
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            print("Server shutting down ...")
            httpd.shutdown()

    return None


if __name__ == '__main__':
    bestatic_serv()



def generator(**config):
    import os
    from datetime import datetime
    from jinja2 import Environment, PackageLoader
    from markdown import markdown
    from markdown.extensions import codehilite
    from markdown.extensions.toc import slugify
    import frontmatter
    import shutil
    import copy
    import re
    from bs4 import BeautifulSoup
    from feedgen.feed import FeedGenerator
    import pytz
    import json
    from bestatic import bestaticSitemap

    def isolate_tags(taglist):
        taglist_2 = re.split(r'\s|(?<!\d)[,.]|,.', taglist)
        taglist_3 = [tag for tag in taglist_2 if tag]
        taglist_final = list(set(taglist_3))
        return taglist_final

    def split_dict_into_n(d, n):
        """
        Splits a dictionary into n different dictionaries.

        Args:
            d (dict): The input dictionary.
            n (int): Number of dictionaries to split into.

        Returns:
            List[dict]: A list of n dictionaries.
        """
        # Calculate the chunk size for each sub-dictionary
        chunk_size = len(d) // n

        # Initialize an empty list to store the sub-dictionaries
        result = []

        # Iterate n times to create n sub-dictionaries
        for i in range(n):
            start_idx = i * chunk_size
            end_idx = (i + 1) * chunk_size if i < n - 1 else len(d)
            sub_dict = dict(list(d.items())[start_idx:end_idx])
            result.append(sub_dict)

        return result

    def json_data_processing(dict_all, json_path):
        json_data_temp = json.dumps(dict_all, indent=2)
        with open(json_path, "w") as filej:
            filej.write(json_data_temp)
        return None

    class Parsing:
        def __init__(self, path_of_md):
            self.path_of_md = path_of_md
            self.metadata = None
            self.content = None
            self.summary = None
            self.tags = None
            self.katex = None
            self.text = None
            self.title = None
            self.slug = None
            self.parse_data()

        def parse_data(self):
            with open(self.path_of_md, 'r', encoding='utf-8') as f:
                self.content = markdown(f.read(), extensions=[
                    "meta", "attr_list", "tables", codehilite.CodeHiliteExtension(linenos="inline"), "fenced_code"])
                f.seek(0)
                self.metadata = frontmatter.load(f).metadata
                plain_text = ''.join(BeautifulSoup(self.content, 'html.parser').findAll(string=True))
                self.text = plain_text
                self.summary = plain_text[:250] + "..." if len(plain_text) > 250 else plain_text
                self.title = self.metadata["title"]
                self.slug = self.metadata["slug"] if "slug" in self.metadata else slugify(self.title, separator="-")
                if "tags" in self.metadata:
                    self.tags = isolate_tags(self.metadata["tags"])
                else:
                    None
                if "katex" in self.metadata and "katex":
                    self.katex = True

    siteURL = config["siteURL"] if config and "siteURL" in config else "https://example.org"
    site_title = config["title"] if config and "title" in config else "A Demo Site for Bestatic"
    site_description = config["description"] if config and "description" in config else "A Demo Site for Bestatic"
    theme_name = config["theme"] if config and "theme" in config else "Amazing"
    rss_feed = config["rss_feed"] \
        if config and "rss_feed" in config else True
    current_directory = os.getcwd()

    shutil.rmtree(os.path.join(current_directory, "_output")) if os.path.exists(
        os.path.join(current_directory, "_output")) else None

    working_directory = os.path.join(current_directory, "themes", theme_name)
    format_files = False if config and "ugly_url" in config and config["ugly_url"] is True else True
    extension = "" if format_files else ".html"
    json_extension = ""

    source = os.path.join(working_directory, "static")
    destination = os.path.join(current_directory, "_output", "static")

    try:
        shutil.copytree(source, destination, dirs_exist_ok=True)
    except shutil.SameFileError:
        print("Source and destination represents the same file.")
    except PermissionError:
        print("Permission denied.")

    POSTS = {}
    PAGES = {}

    if os.path.isdir('posts') and len(os.listdir('posts')):
        for post_counter in os.listdir('posts'):
            input_post_path = os.path.join('posts', post_counter)
            POSTS[post_counter] = Parsing(input_post_path)

    if os.path.isdir('pages') and len(os.listdir('pages')):
        for page_counter in os.listdir('pages'):
            input_page_path = os.path.join('pages', page_counter)
            PAGES[page_counter] = Parsing(input_page_path)

    navigation_items = {key: {"title": value.title, "slug": value.slug} for key, value in PAGES.items() if
                        key != "404.md"}

    env = Environment(loader=PackageLoader("bestatic.generator", os.path.join(working_directory, "templates")))

    env.trim_blocks = True
    env.lstrip_blocks = True

    home_template = None
    page_template = None
    post_template = None
    error_template = None
    list_template = None
    tags_template = None

    if os.path.exists(os.path.join(working_directory, "templates", "home.html.jinja2")):
        home_template = env.get_template('home.html.jinja2')
        home_final = home_template.render(title=site_title, description=site_description,
                                          typeof="home", extension=extension, navi=navigation_items)
        with open(f"_output/index.html", "w", encoding="utf-8") as file:
            file.write(home_final)

    if os.path.isdir('pages') and len(os.listdir('pages')):
        try:
            page_template = env.get_template('page.html.jinja2')
        except FileNotFoundError:
            raise Exception(
                "The 'page.html.jinja2' template must exist in the theme directory to process static pages.")
    if os.path.exists(os.path.join(working_directory, "templates", "404.html.jinja2")):
        error_template = env.get_template('404.html.jinja2')

    if page_template:
        for page in PAGES:

            output_page_path = '_output/{slug}{extension}'.format(slug=PAGES[page].slug, extension=extension)

            if PAGES[page].metadata['slug'] == 404 and error_template:
                page_final = error_template.render(title=site_title, description=site_description,
                                                   typeof="pages", extension=extension, navi=navigation_items)
            else:
                page_final = page_template.render(title=site_title, description=site_description,
                                                  page=PAGES[page], typeof="pages", extension=extension,
                                                  navi=navigation_items)

            os.makedirs(os.path.dirname(output_page_path), exist_ok=True)
            with open(output_page_path, 'w', encoding="utf-8") as file:
                file.write(page_final)


    if os.path.isdir('posts') and len(os.listdir('posts')):
        try:
            post_template = env.get_template('post.html.jinja2')
            list_template = env.get_template('list.html.jinja2')
            tags_template = env.get_template('taglist.html.jinja2')
        except FileNotFoundError:
            raise Exception("The 'post', 'list', and 'taglist' template must exist in the templates directory of "
                            "theme root directory to process the blog/posts/news items.")
    else:
        pass

    if post_template:

        POSTS_SORTED_LIST = sorted(POSTS, key=lambda sorter: datetime.strptime(POSTS[sorter].metadata["date"],
                                                                               "%B %d, %Y"), reverse=True)
        # POSTS_SORTED = {}
        POSTS_SORTED = {item: POSTS[item] for item in POSTS_SORTED_LIST}
        # for item in POSTS_SORTED_LIST:
        #     POSTS_SORTED[item] = POSTS[item]

        POSTS_SORTED_temp = copy.deepcopy(POSTS_SORTED)
        POSTS_SORTED_temp.pop(next(iter(POSTS_SORTED_temp)))

        next_slugs_list = []
        for post in POSTS_SORTED_temp:
            next_slugs_list.append(POSTS_SORTED[post].metadata["slug"])

        prev_slug = None

        user_input_n = config['number_of_pages'] if config and "number_of_pages" in config else 4
        disqus = config["disqus"]["disqusShortname"] \
            if config and "disqus" in config and config["disqus"]["enabled"] is True else False

        for ii, (post, value) in enumerate(POSTS_SORTED.items()):
            output_post_path = '_output/post/{slug}{extension}'.format(slug=POSTS_SORTED[post].slug,
                                                                       extension=extension)
            tags_in_post_individual = POSTS_SORTED[post].tags
            next_slug = next_slugs_list[ii] if ii < len(next_slugs_list) else None
            post_final = post_template.render(title=site_title, description=site_description,
                                              post=POSTS_SORTED[post], typeof="posts", next_slug=next_slug,
                                              prev_slug=prev_slug,
                                              disqus=disqus, extension=extension, navi=navigation_items)
            prev_slug = POSTS_SORTED[post].slug
            os.makedirs(os.path.dirname(output_post_path), exist_ok=True)
            with open(output_post_path, 'w', encoding="utf-8") as file:
                file.write(post_final)

        split_dicts = split_dict_into_n(POSTS_SORTED, user_input_n)

        for jj in range(len(split_dicts)):
            list_final = list_template.render(title=site_title, description=site_description,
                                              post=split_dicts[jj], page_index=jj, page_range=len(split_dicts),
                                              typeof="lists",
                                              extension=extension, navi=navigation_items)
            paginator = "_output/posts" + str(jj + 1) + extension if jj != 0 else "_output/posts" + extension
            with open(paginator, 'w', encoding="utf-8") as file:
                file.write(list_final)

        Tag_list = [POSTS_SORTED[item].metadata["tags"] for item in POSTS_SORTED]
        Tag_list_temp = " ".join(Tag_list)

        Tag_list_final = isolate_tags(Tag_list_temp)

        for tags in Tag_list_final:
            output_tag_path = f'_output/post/tags/{tags}{extension}'
            POSTS_tag = {}
            for item in POSTS_SORTED:
                tags_in_post_correct = POSTS_SORTED[item].tags

                if tags in tags_in_post_correct:
                    POSTS_tag[item] = POSTS[item]

            tag_page_final = tags_template.render(title=site_title, description=site_description,
                                                  post=POSTS_tag, typeof="tags", tags=tags, extension=extension,
                                                  navi=navigation_items)
            os.makedirs(os.path.dirname(output_tag_path), exist_ok=True)
            with open(output_tag_path, 'w', encoding="utf-8") as file:
                file.write(tag_page_final)

    json_combined_dict = {}

    json_dict_post = {key: {'title': value.title, 'text': value.text, 'slug': f"post/{value.slug}"} for key, value in
                      POSTS.items()} if post_template else {}

    json_dict_page = {key: {'title': value.title, 'text': value.text, 'slug': value.slug} for key, value in
                      PAGES.items() if key != "404.md"} if page_template else {}

    json_combined_dict = {**json_dict_post, **json_dict_page}

    if json_combined_dict:
        result_dict = [{'uri': f"{value['slug']}{json_extension}", 'title': value['title'], 'content': value['text']}
                       for
                       value
                       in
                       json_combined_dict.values()]

        result_dict_post = [
            {'uri': f"../{value['slug']}{json_extension}", 'title': value['title'], 'content': value['text']} for
            value in
            json_combined_dict.values()]

        result_dict_tags = [
            {'uri': f"../../{value['slug']}{json_extension}", 'title': value['title'], 'content': value['text']} for
            value in
            json_combined_dict.values()]

        json_data_processing(result_dict, f'_output/index.json')

        if post_template:
            json_data_processing(result_dict_post, f'_output/post/index.json')
            json_data_processing(result_dict_tags, f'_output/post/tags/index.json')

    bestaticSitemap.generate_sitemap(siteURL, "_output")

    timezone = pytz.timezone('UTC')

    if post_template and rss_feed is True:
        rss_dict_post = {
            key: {'title': value.title, 'text': value.text, 'slug': f"post/{value.slug}",
                  'date': datetime.strptime(value.metadata["date"], "%B %d, %Y")} for
            key, value in
            reversed(POSTS_SORTED.items())} if post_template else {}

        posts_dict = [
            {'uri': f"{siteURL}/{value['slug']}{json_extension}", 'title': value['title'], 'content': value['text'],
             'date': timezone.localize(value['date'])} for
            value
            in
            rss_dict_post.values()]

        fg = FeedGenerator()
        fg.title(site_title)
        fg.link(href=siteURL + "/posts", rel='alternate')
        fg.description(site_description)

        # Add entries from the dictionary
        for entry_data in posts_dict:
            entry = fg.add_entry()
            entry.title(entry_data['title'])
            entry.description(entry_data['content'])
            entry.link(href=entry_data['uri'])
            entry.pubDate(entry_data['date'])

        # Generate the RSS feed
        rss_feed = fg.rss_str(pretty=True)

        # Save the feed to a file (optional)
        with open('_output/index.rss', 'wb') as rss_file:
            rss_file.write(rss_feed)

    return None


if __name__ == '__main__':
    generator()
